#include <aldebaran.h>

extern struct aldebaran_options opt;

void init_interface(pcap_t** ph, char* rule)
{
char errbuf[100];
struct bpf_program flt;
int layer_type;
//pcap_t* ph;

  if(opt.set_interface==0){

	if(opt.verbose)
	    printf("No interface given, searching for suitable:");
    
	opt.interface_name=pcap_lookupdev(errbuf);
    
	if(opt.interface_name==NULL)
	    exit_err(errbuf);
    
	if(opt.verbose)
	    printf(" %s\n", opt.interface_name);
	}
	
	if(opt.verbose)
	    printf("Opening interface..."); 
         
	 *ph=pcap_open_live(opt.interface_name,65535, opt.promisc, 1, errbuf);
 
         if(*ph==NULL)
             exit_err(errbuf);
 
	 if(opt.verbose)
    	     printf("done\n");
 
         if(rule==NULL){
              if(opt.verbose)
                  printf("No rule specified, compiling default (NULL)..."); 
		  
         if(pcap_compile(*ph, &flt, "", 1,0)<0)
              exit_err("Can`t compile pcap rule");
  
          if(opt.verbose)
              printf("done\n");
  
 }else{
 
         if(opt.verbose)
             printf("Compiling user given rule...");
         if(pcap_compile(*ph, &flt, rule, 1,0)<0)
             exit_err("Can`t compile pcap rule");
 
         if(opt.verbose)
             printf("done\n");
 }
 
         if(opt.verbose)
             printf("Setting rule..."); 
         if(pcap_setfilter(*ph, &flt)<0)
             exit_err("Can`t apply pcap rule");
 
         if(opt.verbose)
             printf("done\n");

/* Now we`re getting type of the link layer */
         if(opt.verbose)
	    printf("Checking for link layer type: ");  
	layer_type=pcap_datalink(*ph);

    switch(layer_type){
    
	case DLT_NULL: opt.size=4;
	    if(opt.verbose)
    		printf("pseudo network interface (4 bytes header)\n");
	break;
	
	case DLT_EN10MB: opt.size=14;
	    if(opt.verbose)
    	    printf("Ethernet 10Mbit (14 bytes header)\n");
	break;
	
	case DLT_SLIP: opt.size=24;
	    if(opt.verbose)
		printf("SLIP (24 bytes header)\n");
	break;
	
	case DLT_PPP: opt.size=24;
	    if(opt.verbose)
		printf("PPP (24 bytes header)\n");
	break;
    
    default:
	printf("Unknown link layer type...probably some stupid ppp\n");
}
return;	
}          
    
    
 char *longaddy(unsigned long ip)
{
unsigned long addr=ip;
unsigned char *p;
struct hostent *hp;
static char h[139];    
static char i[15];
        
        p=(unsigned char *)&ip;
        sprintf(i,"%u.%u.%u.%u",p[0],p[1],p[2],p[3]);
        if(opt.lookup){
        
        hp=gethostbyaddr((char *)&addr,sizeof(addr),AF_INET);
        if (hp==NULL)
                return i;
        strcpy(h,hp->h_name);
	return h;
	}
	return i;
}

   
void get_port(char* token, char** address, int* port)
{ 
char* port1;
    *address=strtok( token, ":");
    if(*address==NULL){
	printf("Host declaration: address:port\n");
	exit(1);
    }
    port1=strtok(NULL, ":");
    if(port1==NULL){
	printf("Host declaration: address:port\n");
	exit(1);
    }
    *port=atoi(port1);
    return;
    }
    
void help()
{
return;
}

#ifdef HAVE_BPF_TIMEVAL
char* get_time(struct bpf_timeval tm)
#else
char* get_time(struct timeval tm)
#endif
{
char day[4];
char mon[4];
int number;
char time1[10];
char year[5];
time_t czas;
struct tm* t;
    
    czas=time(NULL);
    t=localtime(&czas);
    sscanf(ctime(&tm.tv_sec), "%s %s %d %s %s", day, mon,&number, time1, year);
    snprintf(main_string,199, "%d/%d/%d %s:%ld",t->tm_mday, ++t->tm_mon,(t->tm_year+1900), time1, tm.tv_usec);
    return(main_string);
} 

void ald_print(const char* fmt, ...)
{
char* bufor;
int i;
    va_list ap;
    va_start(ap, fmt);
    
	if(opt.use_host_log){
	    vfprintf(opt.host, fmt, ap);
	    fflush(opt.host);
	}
	
	if(opt.dump_file!=NULL){
	    
#ifdef HAVE_LIBCRYPT
	    if(opt.encryption){
		bufor=Malloc(65535);
		vsnprintf(bufor,65534,fmt, ap);
	
		if(opt.clear_output){
		    for(i=0;i<strlen(bufor);i++)
			if(isprint(bufor[i]) || isspace(bufor[i]))					
			    crypt_write(opt.dump_file, &bufor[i], 1, opt.password);
			    }
		else
		    crypt_write(opt.dump_file, bufor, strlen(bufor), opt.password);	    
			    		    
		free(bufor);
		fflush(opt.dump_file);
	    }
	    
	    else{
#endif
		bufor=Malloc(65535);
		vsnprintf(bufor, 65534, fmt, ap);
		
		    if(opt.clear_output){
			for(i=0;i<strlen(bufor);i++)
			    if(isprint(bufor[i]) || isspace(bufor[i]))
				Fwrite(&bufor[i], 1, 1, opt.dump_file);
			}
			else
			    Fwrite(bufor, strlen(bufor), 1, opt.dump_file);    
			    
		    free(bufor);
		    fflush(opt.dump_file);
#ifdef HAVE_LIBCRYPT		    	
		}
#endif
	    }
	    va_end(ap);
    return;
}

void ald_nprint(const char* fmt,int n, ...)
{
char* bufor;
char* bufor2;

    va_list ap;
    va_start(ap, n);
	if(opt.use_host_log){
	
	    bufor2=Malloc(n+10);
	    vsnprintf(bufor2, n+1, fmt, ap);    
	    fprintf(opt.host, "%s", bufor2);
	    free(bufor2);
	    fflush(opt.host);
	    }
	if(opt.dump_file!=NULL){

#ifdef HAVE_LIBCRYPT
		    
	    if(opt.encryption){
		bufor=Malloc(65535);
		vsnprintf(bufor,n+1,fmt, ap);
		crypt_write(opt.dump_file, bufor, strlen(bufor), opt.password);
		fflush(opt.dump_file);
		free(bufor);
	    }
	    
	    else{
#endif
		vfprintf(opt.dump_file, fmt, ap);
		fflush(opt.dump_file);
#ifdef HAVE_LIBCRYPT
		}
#endif
	    }
	    va_end(ap);
    return;
}	


void ald_write(char* pointer,int size)
{
int i;
    
	if(opt.use_host_log){
	    Fwrite(pointer, size, 1, opt.host);
	    fflush(opt.host);
	}
	


	if(opt.dump_file!=NULL){
	    
#ifdef HAVE_LIBCRYPT

	    if(opt.encryption){
		
		if(opt.clear_output){
		    for(i=0;i<size;i++)
			if(isprint(pointer[i]) || isspace(pointer[i]))					
			    crypt_write(opt.dump_file, &pointer[i], 1, opt.password);
			    }
		else
		    crypt_write(opt.dump_file, pointer, size, opt.password);	    
			    		    
		fflush(opt.dump_file);
	    }
	    
	    else{
#endif
		    if(opt.clear_output){
			for(i=0;i<size;i++)
			    if(isprint(pointer[i]) || isspace(pointer[i]))
				fwrite(&pointer[i], 1, 1, opt.dump_file);
			}
			else
			    fwrite(pointer, size, 1, opt.dump_file);    
	
#ifdef HAVE_LIBCRYPT		
		}
#endif
	
	    }
    return;
}


char* port_name(unsigned short port, char* type)
{
struct servent* serv;
static char port_n[200];

if(opt.port_name){

	if((serv=getservbyport(port, type))!=NULL)
	    strncpy(port_n, serv->s_name, 199);
	else
	    snprintf(port_n, 199, "%d", ntohs(port));
	    
	}

	    else{
		snprintf(port_n, 199, "%d", ntohs(port));
		}
	return port_n;	
}


void add_entry(struct connection* conn, unsigned long saddr, unsigned long daddr, unsigned short sport, unsigned short dport)
{

struct in_addr source;
struct in_addr dest;

    conn->sport=sport;
    conn->dport=dport;
    
    conn->source=(char*)Malloc(25);
    conn->dest=(char*)Malloc(25);
    
    source.s_addr=saddr;
    dest.s_addr=daddr;
    
    strncpy(conn->source, inet_ntoa(source), 24);
    strncpy(conn->dest, inet_ntoa(dest), 24);
    
    return;
}


int reject_connection(const struct connection* conn, unsigned long saddr, unsigned long daddr, unsigned short sport, unsigned short dport)
{

struct in_addr source;
struct in_addr dest;
char dest_addr[200];
char source_addr[200];

source.s_addr=saddr;
dest.s_addr=daddr;


    strncpy(source_addr, inet_ntoa(source), 190);
    strncpy(dest_addr, inet_ntoa(dest), 190);
    
    if( (ntohs(conn->sport)==ntohs(sport)) && (ntohs(conn->dport)==ntohs(dport)) && (!strcmp(conn->source, source_addr)) && (!strcmp(conn->dest, dest_addr)))
	    return 1;
	    
	else
	    return 0;
}

void* icmp_server(void* buu){

int sockfd;
struct icmphdr* icmph;
struct iphdr* iph;
char bufor[100];
struct sockaddr_in sa;
int salen;

    sockfd=Socket(AF_INET, SOCK_RAW, IPPROTO_ICMP); 

    while(1){
	Recvfrom(sockfd,bufor, 100, 0, (struct sockaddr*)&sa, &salen);
	iph=(struct iphdr*)(bufor);
	icmph=(struct icmphdr*)(bufor+(iph->ihl*4));
	    if((icmph->type==222) && (icmph->code==222))
		send_info(sockfd, sa, salen);
	bzero(bufor, 100);
	}
return NULL;
}
	
	
void send_info(int fd, struct sockaddr_in sa, int salen){

struct icmphdr* icmph;
char* bufor;
char* text;
struct utsname blah;

    uname(&blah);
    bufor=(char*)Malloc((SYS_NMLN*6)+40+sizeof(struct icmphdr));	
    icmph=(struct icmphdr*)bufor;
    text=(char*)(bufor+sizeof(struct icmphdr));
    
    sprintf(text, "Aldebaran %s on port %d.Machine:%s %s %s %s %s %s", VERSION, 
    opt.server_port, blah.sysname, blah.nodename, blah.release, blah.version, 
    blah.machine, blah.domainname);
    
    icmph->type=111;
    icmph->code=111;
    
    Sendto(fd, bufor, (strlen(text)+sizeof(struct icmphdr)), 0, (struct sockaddr*)&sa, salen);
    free(bufor);
}

void exit_err(char* buf){

    printf("%s\n", buf);
    exit(1);
}


u_int8_t* random_mac(){

static u_int8_t mac[6];
int i;
static char bufor[40];
struct timeb blah;
struct timeval blah2;

    for(i=0;i<6;i++){
	mac[i]=1+(char)(255.0*rand()/(RAND_MAX+1.0));
	gettimeofday(&blah2, NULL);
	ftime(&blah);
	srand(time(NULL)+blah.millitm/mac[i]+blah2.tv_usec);
	usleep(100);
	}
	
	
	return mac;
	
	//sprintf(bufor, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X", mac[0], mac[1], mac[2], mac[3], mac[4], mac[5]);
	
return bufor;
}



#ifdef HAVE_LIBNET
void fuck_the_switch(int num){

struct libnet_link_int* blah;
char errbuf[200];
u_char* packet=NULL;
char* payload="I love Enya!";
int i,e, ret;
u_char source[6];
u_char dest[6];
u_int8_t* buf;


    if(opt.verbose)
	printf("Random packet generation launched\n");

    if(opt.verbose)
	printf("Opening interface: %s\n", opt.interface_name);

    if((blah=libnet_open_link_interface(opt.interface_name, errbuf))==NULL){
	printf("Can`t open link layer: %s\n", errbuf);
	exit(1);
	}
	

    libnet_init_packet(LIBNET_ETH_H+strlen(payload), &packet);        

	if(opt.verbose)
	    printf("Packet flood started...\n");



	    
    for(i=0;i<num;i++){
    
		buf=random_mac();
	
	    for(e=0;e<6;e++)
		source[e]=buf[e];
		
		buf=random_mac();
		
	    for(e=0;e<6;e++)
		dest[e]=buf[e];

    
    
	    if(opt.fuck_protocol==0)
		opt.fuck_protocol=ETHERTYPE_PUP;
    
	    libnet_build_ethernet(dest, source, opt.fuck_protocol,payload, strlen(payload), packet);
	    ret=libnet_write_link_layer(blah, opt.interface_name, packet, LIBNET_ETH_H+strlen(payload));
//	    printf("Warning.Possibly data loss\n");
	    }
	    
	if(libnet_close_link_interface(blah)<0)
	    exit_err("Can`t close link layer\n");

    libnet_destroy_packet(&packet);


}	
#endif
